"""
__version__.py
~~~~~~~~~~~~~~
Information about the current version of the py-package-template package.
"""

__title__ = "insightspy"
__description__ = "Python API for Arpeggio Biosciences data portal"
__version__ = "0.0.4"
__author__ = "Noah Dukler"
__author_email__ = "noah@arpeggiobio.com"
__license__ = "MIT"
__url__ = "https://github.com/ArpeggioBiosciences/insightspy"
__classifiers__ = [
    "Programming Language :: Python :: 3",
    "Operating System :: OS Independent",
]
